"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.typescript = void 0;
var pluginutils_1 = require("@rollup/pluginutils");
var Engine_1 = __importDefault(require("./Engine"));
var utils_1 = require("./utils");
var defaultOptions = {
    include: ['*.ts+(|x)', '**/*.ts+(|x)'],
    exclude: ['*.d.ts', '**/*.d.ts'],
    check: true,
};
function typescript(options) {
    var opts = __assign(__assign({}, defaultOptions), options);
    var filter = pluginutils_1.createFilter(opts.include, opts.exclude);
    var cwd = process.cwd();
    var engine = new Engine_1.default(cwd);
    var host = utils_1.createFormattingHost(engine.getCompilerOptions());
    var resolveModule = utils_1.createModuleResolver(host);
    return {
        name: 'rpt',
        resolveId: function (source, importer) {
            if (source === 'tslib') {
                return source;
            }
            if (!importer)
                return null;
            // Convert path from windows separators to posix separators
            var containingFile = utils_1.normalizePath(importer);
            var resolved = resolveModule(source, containingFile);
            if (resolved) {
                if (resolved.extension === '.d.ts')
                    return null;
                return resolved.resolvedFileName;
            }
            return null;
        },
        transform: function (code, id) {
            if (!filter(id))
                return;
            if (opts.check) {
                return engine.transformWithCheck(id, code);
            }
            else {
                return engine.transform(id, code);
            }
        }
    };
}
exports.typescript = typescript;
exports.default = typescript;
