"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDeclareFile = exports.loopEach = exports.createModuleResolver = exports.createFormattingHost = exports.normalizePath = void 0;
var path_1 = __importDefault(require("path"));
var fs_1 = __importDefault(require("fs"));
var typescript_1 = __importDefault(require("typescript"));
function normalizePath(fileName) {
    return fileName.split(path_1.default.win32.sep).join(path_1.default.posix.sep);
}
exports.normalizePath = normalizePath;
function createFormattingHost(compilerOptions) {
    return {
        /** Returns the compiler options for the project. */
        getCompilationSettings: function () { return compilerOptions; },
        /** Returns the current working directory. */
        getCurrentDirectory: function () { return process.cwd(); },
        /** Returns the string that corresponds with the selected `NewLineKind`. */
        getNewLine: function () {
            switch (compilerOptions.newLine) {
                case typescript_1.default.NewLineKind.CarriageReturnLineFeed:
                    return '\r\n';
                case typescript_1.default.NewLineKind.LineFeed:
                    return '\n';
                default:
                    return typescript_1.default.sys.newLine;
            }
        },
        /** Returns a lower case name on case insensitive systems, otherwise the original name. */
        getCanonicalFileName: function (fileName) {
            return typescript_1.default.sys.useCaseSensitiveFileNames ? fileName : fileName.toLowerCase();
        }
    };
}
exports.createFormattingHost = createFormattingHost;
function createModuleResolver(host) {
    var compilerOptions = host.getCompilationSettings();
    var cache = typescript_1.default.createModuleResolutionCache(process.cwd(), host.getCanonicalFileName, compilerOptions);
    var moduleHost = __assign(__assign({}, typescript_1.default.sys), host);
    return function (moduleName, containingFile) {
        var resolved = typescript_1.default.nodeModuleNameResolver(moduleName, containingFile, compilerOptions, moduleHost, cache);
        return resolved.resolvedModule;
    };
}
exports.createModuleResolver = createModuleResolver;
function loopEach(path, callback) {
    if (!fs_1.default.existsSync(path))
        return;
    var stats = fs_1.default.statSync(path);
    if (stats.isDirectory()) {
        var dir = fs_1.default.readdirSync(path);
        for (var i = 0; i < dir.length; i += 1) {
            var curPath = path + "/" + dir[i];
            var curStats = fs_1.default.statSync(curPath);
            if (curStats.isDirectory()) {
                loopEach(curPath, callback);
            }
            else if (curStats.isFile()) {
                callback(curPath);
            }
        }
    }
    else {
        callback(path);
    }
}
exports.loopEach = loopEach;
function getDeclareFile(cwd) {
    if (!fs_1.default.existsSync(cwd))
        return [];
    var status = fs_1.default.statSync(cwd);
    var dir = fs_1.default.readdirSync(cwd);
    var result = [];
    for (var i = 0; i < dir.length; i += 1) {
        if (!/node_modules/.test(dir[i])) {
            loopEach(cwd + "/" + dir[i], function (url) {
                if (url.endsWith('.d.ts')) {
                    result.push(url);
                }
            });
        }
    }
    return result;
}
exports.getDeclareFile = getDeclareFile;
